@@ -54,8 +54,8 @@ import java.util.*;
  * That way, multiple threads can have multiple contexts current and render to them concurrently.
  *
  * @author elias_naur <elias_naur@users.sourceforge.net>
- * @version $Revision: 3279 $
- *          $Id: GLContext.java 3279 2010-03-11 21:06:49Z spasi $
+ * @version $Revision: 3355 $
+ *          $Id: GLContext.java 3355 2010-05-27 22:56:29Z spasi $
  */
 public final class GLContext {
 
@@ -169,6 +169,22 @@ public final class GLContext {
 		return 0;
 	}
 
+	/**
+	 * Helper method to get a pointer to a named function with aliases in the OpenGL library.
+	 *
+	 * @param aliases the function name aliases.
+	 *
+	 * @return the function pointer address
+	 */
+	static long getFunctionAddress(String[] aliases) {
+		for ( int i = 0; i < aliases.length; i++ ) {
+			long address = getFunctionAddress(aliases[i]);
+			if ( address != 0 )
+				return address;
+		}
+		return 0;
+	}
+
 	/** Helper method to get a pointer to a named function in the OpenGL library */
 	static native long getFunctionAddress(String name);
 
