@@ -39,6 +39,7 @@ import java.awt.FontMetrics;
 import java.awt.Graphics;
 import java.awt.Image;
 import java.awt.MediaTracker;
+import java.awt.image.ImageObserver;
 import java.io.DataInputStream;
 import java.io.DataOutputStream;
 import java.io.File;
@@ -47,6 +48,8 @@ import java.io.FileOutputStream;
 import java.io.FilePermission;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.ObjectInputStream;
+import java.io.ObjectOutputStream;
 import java.io.OutputStream;
 import java.io.PrintWriter;
 import java.io.StringWriter;
@@ -68,6 +71,7 @@ import java.security.PrivilegedExceptionAction;
 import java.security.SecureClassLoader;
 import java.security.cert.Certificate;
 import java.util.Enumeration;
+import java.util.HashMap;
 import java.util.StringTokenizer;
 import java.util.Vector;
 import java.util.jar.JarEntry;
@@ -106,11 +110,17 @@ import sun.security.util.SecurityConstants;
  * <ul>
  * <li>al_version - [int or float] Version of deployment. If this is specified, the jars will be cached and 
  * reused if the version matches. If version doesn't match all of the files are reloaded.</li>
- * <li>al_bgcolor - [String] Hex formated color to use as background. <i>Default: ffffff</i>.</li>
- * <li>al_fgcolor - [String] Hex formated color to use as foreground. <i>Default: 000000</i>.</li>
- * <li>al_errorcolor - [String] Hex formated color to use as foreground color on error. <i>Default: ff0000</i>.</li>
+ * <li>al_cache - [boolean] Whether to use cache system. <i>Default: true</i>.</li>
  * <li>al_debug - [boolean] Whether to enable debug mode. <i>Default: false</i>.</li>
  * <li>al_prepend_host - [boolean] Whether to limit caching to this domain, disable if your applet is hosted on multple domains and needs to share the cache. <i>Default: true</i>.</li>
+ * <ul>
+ * <li>al_windows64 - [String] If specified it will be used instead of al_windows on 64bit windows systems.</li>
+ * <li>al_windows32 - [String] If specifed it will be used instead of al_windows on 32bit windows systems.</li>
+ * <li>al_linux64 - [String] If specifed it will be used instead of al_linux on 64bit linux systems.</li>
+ * <li>al_linux32 - [String] If specifed it will be used instead of al_linux on 32bit linux systems.</li>
+ * <ul>
+ * <li>boxbgcolor - [String] any String AWT color ("red", "blue", etc), RGB (0-255) or hex formated color (#RRGGBB) to use as background. <i>Default: #ffffff</i>.</li>
+ * <li>boxfgcolor - [String] any String AWT color ("red", "blue", etc), RGB (0-255) or hex formated color (#RRGGBB) to use as foreground. <i>Default: #000000</i>.</li>
  * </ul>
  * </p>
  * @author kappaOne
@@ -166,19 +176,19 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 	protected int		totalSizeExtract; 
 	
 	/** logo to be shown while loading */
-	protected Image		logo;
+	protected Image		logo, logoBuffer;
 
 	/** progressbar to render while loading */
-	protected Image		progressbar;
+	protected Image		progressbar, progressbarBuffer;
 	
 	/** offscreen image used */
 	protected Image 	offscreen;
 	
+	/** set to true while painting is done */
+	protected boolean 	painting;
+	
 	/** background color of applet */
 	protected Color		bgColor 	= Color.white;
-	
-	/** Color to write errors in */
-	protected Color		errorColor 	= Color.red;
 
 	/** color to write foreground in */
 	protected Color		fgColor 	= Color.black;	
@@ -210,6 +220,15 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 	/** whether to prepend host to cache path */
 	protected boolean 	prependHost;
 	
+	/** Used to store file names with lastModified time */
+	protected HashMap 	filesLastModified;
+	
+	/** Sizes of files to download */
+	protected int[] 	fileSizes;
+	
+	/** whether to use caching system, only download files that have changed */
+	protected boolean 	cacheEnabled;
+	
 	/** String to display as a subtask */
 	protected String	subtaskMessage = "";
 
@@ -252,7 +271,10 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 				return;				
 			}
 		}
-
+		
+		// whether to use cache system
+		cacheEnabled	= getBooleanParameter("al_cache", true);
+		
 		// whether to run in debug mode
 		debugMode 		= getBooleanParameter("al_debug", false);
 		
@@ -260,11 +282,9 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 		prependHost 	= getBooleanParameter("al_prepend_host", true);
 		
 		// get colors of applet
-		bgColor 		= getColor("al_bgcolor", Color.white);
+		bgColor 		= getColor("boxbgcolor", Color.white);
 		setBackground(bgColor);
-
-		fgColor 		= getColor("al_fgcolor", Color.black);
-		errorColor 		= getColor("al_errorcolor", Color.red);		
+		fgColor 		= getColor("boxfgcolor", Color.black);	
 
 		// load logos
 		logo 			= getImage(getParameter("al_logo"));
@@ -339,7 +359,6 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 		if (lwjglApplet != null) {
 			lwjglApplet.stop();
 		}
-		super.stop();
 	}
 	
 	/*
@@ -353,7 +372,8 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 		progressbar = null;
 		logo 		= null;
 		
-		super.destroy();
+		logoBuffer = null;
+		progressbarBuffer = null;
 	}
 	
 	/**
@@ -390,22 +410,34 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 		// create offscreen if missing
 		if (offscreen == null) {
 			offscreen = createImage(getWidth(), getHeight());
+			
+			// create buffers for animated gifs
+			logoBuffer = createImage(logo.getWidth(null), logo.getHeight(null));
+			progressbarBuffer = createImage(logo.getWidth(null), logo.getHeight(null));
+			
+			// add image observer, it will notify when next animated gif frame is ready 
+			offscreen.getGraphics().drawImage(logo, 0, 0, this);
+			offscreen.getGraphics().drawImage(progressbar, 0, 0, this);
+			
+			// in case image is not animated fill image buffers once
+			imageUpdate(logo, ImageObserver.FRAMEBITS, 0, 0, 0, 0);
+			imageUpdate(progressbar, ImageObserver.FRAMEBITS, 0, 0, 0, 0);
 		}
 
 		// draw everything onto an image before drawing to avoid flicker
 		Graphics og = offscreen.getGraphics();
 		FontMetrics fm = og.getFontMetrics();
 
-		// set background color
+		// clear background color
 		og.setColor(bgColor);
-		og.fillRect(0, 0, getWidth(), getHeight());
+		og.fillRect(0, 0, offscreen.getWidth(null), offscreen.getHeight(null));
 
 		// get logo position so its in the middle of applet
 		int x = 0, y = 0;
 		
 		if(logo != null && !fatalError) {
-			x = (getWidth() - logo.getWidth(this)) / 2;
-			y = (getHeight() - logo.getHeight(this)) / 2;
+			x = (offscreen.getWidth(null) - logo.getWidth(null)) / 2;
+			y = (offscreen.getHeight(null) - logo.getHeight(null)) / 2;
 		}
 
 		og.setColor(fgColor);
@@ -421,41 +453,79 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 			
 			for(int i=0; i<errorMessage.length; i++) {
 				if(errorMessage[i] != null) {
-					int messageX = (getWidth() - fm.stringWidth(errorMessage[i])) / 2;
-					int messageY = (getHeight() - (fm.getHeight() * errorMessage.length)) / 2;
+					int messageX = (offscreen.getWidth(null) - fm.stringWidth(errorMessage[i])) / 2;
+					int messageY = (offscreen.getHeight(null) - (fm.getHeight() * errorMessage.length)) / 2;
 					
-					og.setColor(errorColor);
 					og.drawString(errorMessage[i], messageX, messageY + i*fm.getHeight());
 				}
 			}
 		} else {
 			og.setColor(fgColor);
-
+			
+			painting = true;
+			
 			// draw logo
-			og.drawImage(logo, x, y, null);
+			og.drawImage(logoBuffer, x, y, this);
 			
 			// draw message
-			int messageX = (getWidth() - fm.stringWidth(message)) / 2;
-			int messageY = y + logo.getHeight(null) + 20;
+			int messageX = (offscreen.getWidth(null) - fm.stringWidth(message)) / 2;
+			int messageY = y + logoBuffer.getHeight(null) + 20;
 			og.drawString(message, messageX, messageY);
 			
 			// draw subtaskmessage, if any
 			if(subtaskMessage.length() > 0) {
-				messageX = (getWidth() - fm.stringWidth(subtaskMessage)) / 2;
+				messageX = (offscreen.getWidth(null) - fm.stringWidth(subtaskMessage)) / 2;
 				og.drawString(subtaskMessage, messageX, messageY+20);
 			}
 
 			// draw loading bar, clipping it depending on percentage done
-			int barSize = (progressbar.getWidth(this) * percentage) / 100;
-			og.clipRect(0, 0, x + barSize, getHeight());
-			og.drawImage(progressbar, x, y, null);
+			int barSize = (progressbarBuffer.getWidth(null) * percentage) / 100;
+			og.clipRect(0, 0, x + barSize, offscreen.getHeight(null));
+			og.drawImage(progressbarBuffer, x, y, this);
+			
+			painting = false;
 		}
 		
 		og.dispose();
-
-		// finally draw it all
-		g.drawImage(offscreen, 0, 0, null);
-	}	
+		
+		// finally draw it all centred
+		g.drawImage(offscreen, (getWidth() - offscreen.getWidth(null))/2, (getHeight() - offscreen.getHeight(null))/2, null);
+	}
+	
+	/**
+	 * When an animated gif frame is ready to be drawn the ImageObserver 
+	 * will call this method.
+	 * 
+	 * The Image frame is copied into a buffer, which is then drawn. 
+	 * This is done to prevent image tearing on gif animations.
+	 */
+	public boolean imageUpdate(Image img, int flag, int x, int y, int width, int height) {
+		
+		// if image frame is ready to be drawn and is currently not being painted
+		if (flag == ImageObserver.FRAMEBITS && !painting) {
+			Image buffer;
+			
+			// select which buffer to fill
+			if (img == logo) buffer = logoBuffer;
+			else buffer = progressbarBuffer;
+			
+			Graphics g = buffer.getGraphics();
+			
+			// clear background on buffer
+			g.setColor(bgColor);
+			g.fillRect(0, 0, buffer.getWidth(null), buffer.getHeight(null));
+			
+			// buffer background is cleared, so draw logo under progressbar
+			if (img == progressbar) g.drawImage(logoBuffer, 0, 0, null);
+			
+			g.drawImage(img, 0, 0, this);
+			g.dispose();
+			
+			repaint();
+		}
+		
+		return true;
+	}
 
 	/**
 	 * @return string describing the state of the loader
@@ -467,7 +537,7 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 			case STATE_DETERMINING_PACKAGES:
 				return "Determining packages to load";
 			case STATE_CHECKING_CACHE:
-				return "Checking cache for existing files";
+				return "Calculating download size";
 			case STATE_DOWNLOADING:
 				return "Downloading packages";
 			case STATE_EXTRACTING_PACKAGES:
@@ -534,9 +604,31 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 		String nativeJar 	= null;
 		
 		if (osName.startsWith("Win")) {
-			nativeJar = getParameter("al_windows");
+			
+			// check if arch specific natives have been specified
+			if (System.getProperty("os.arch").endsWith("64")) {
+				nativeJar = getParameter("al_windows64");
+			} else {
+				nativeJar = getParameter("al_windows32");
+			}
+			
+			if (nativeJar == null) {
+				nativeJar = getParameter("al_windows");
+			}
+			
 		} else if (osName.startsWith("Linux")) {
-			nativeJar = getParameter("al_linux");
+			
+			// check if arch specific natives have been specified
+			if (System.getProperty("os.arch").endsWith("64")) {
+				nativeJar = getParameter("al_linux64");
+			} else {
+				nativeJar = getParameter("al_linux32");
+			}
+			
+			if (nativeJar == null) {
+				nativeJar = getParameter("al_linux");
+			}
+			
 		} else if (osName.startsWith("Mac")) {
 			nativeJar = getParameter("al_mac");
 		} else if (osName.startsWith("Solaris") || osName.startsWith("SunOS")) {
@@ -598,10 +690,11 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 			if (!dir.exists()) {
 				dir.mkdirs();
 			}
-			dir = new File(dir, "version");
-
-			// if applet already available don't download anything
-			boolean cacheAvailable = false;
+			
+			File versionFile = new File(dir, "version");
+			
+			// if specified applet version already available don't download anything
+			boolean versionAvailable = false;
 
 			// version of applet
 			String version = getParameter("al_version");
@@ -613,10 +706,10 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 				latestVersion = Float.parseFloat(version);
 
 				// if version file exists
-				if (dir.exists()) {
+				if (versionFile.exists()) {
 					// compare to new version
-					if (latestVersion <= readVersionFile(dir)) {
-						cacheAvailable = true;
+					if (latestVersion <= readVersionFile(versionFile)) {
+						versionAvailable = true;
 						percentage = 90;
 						
 						if(debugMode) {
@@ -628,9 +721,12 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 			}
 
 			// if jars not available or need updating download them
-			if (!cacheAvailable) {
+			if (!versionAvailable) {
+				// get jars file sizes and check cache
+				getJarInfo(dir);		// 5-15%
+				
 				// downloads jars from the server
-				downloadJars(path);		// 10-55%
+				downloadJars(path);		// 15-55%
 				
 				// Extract Pack and LZMA files
 				extractJars(path);		// 55-65%
@@ -638,11 +734,14 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 				// Extracts Native Files
 				extractNatives(path);	// 65-85%
 
-				// add version information once jars downloaded successfully
+				// save version information once jars downloaded successfully
 				if (version != null) {
 					percentage = 90;
 					writeVersionFile(dir, latestVersion);
 				}
+				
+				// save file names with last modified info once downloaded successfully
+				writeCacheFile(new File(dir, "cache"), filesLastModified);
 			}
 
 			// add the downloaded jars and natives to classpath
@@ -688,6 +787,33 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 		dos.writeFloat(version);
 		dos.close();
 	}
+	
+	/**
+	 * read the current cache file
+	 * 
+	 * @param file the file to read
+	 * @return the hashmap containing the files names and lastModified times
+	 * @throws Exception if it fails to read hashmap
+	 */
+	protected HashMap readCacheFile(File file) throws Exception {
+		ObjectInputStream dis = new ObjectInputStream(new FileInputStream(file));
+		HashMap hashMap = (HashMap)dis.readObject();
+		dis.close();
+		return hashMap;
+	}
+
+	/**
+	 * write out cache file of applet
+	 * 
+	 * @param file the file to write out to
+	 * @param filesLastModified the hashmap containing files names and lastModified times
+	 * @throws Exception if it fails to write file
+	 */
+	protected void writeCacheFile(File file, HashMap filesLastModified) throws Exception {
+		ObjectOutputStream dos = new ObjectOutputStream(new FileOutputStream(file));
+		dos.writeObject(filesLastModified);
+		dos.close();
+	}
 
 	/**
 	 * Edits the ClassPath at runtime to include the jars
@@ -765,7 +891,7 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 	 * Due to the way applets on plugin1 work, one jvm must
 	 * be used for all applets. We need to use multiple 
 	 * classloaders in the same jvm due to LWJGL's static 
-	 * nature. I order to solver this we simply remove the 
+	 * nature. I order to solve this we simply remove the 
 	 * natives from a previous classloader allowing a new 
 	 * classloader to use those natives in the same jvm.
 	 * 
@@ -830,7 +956,69 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 		state = STATE_START_REAL_APPLET;
 		lwjglApplet.start();
 	}
-
+	
+	/**
+	 * This method will get the files sizes of the files to download.
+	 * It wil further get the lastModified time of files
+	 * and save it in a hashmap, if cache is enabled it will mark
+	 * those files that have not changed since last download to not
+	 * redownloaded.
+	 * 
+	 * @param dir - location to read cache file from
+	 * @throws Exception - if fails to get infomation
+	 */
+	protected void getJarInfo(File dir) throws Exception {
+		
+		filesLastModified = new HashMap();
+		
+		// store file sizes and mark which files not to download
+		fileSizes = new int[urlList.length];
+		
+		URLConnection urlconnection;
+		
+		File cacheFile = new File(dir, "cache");
+		
+		// if cache file exists, load it
+		if (cacheFile.exists()) {
+			filesLastModified = readCacheFile(cacheFile);
+		}
+		
+		// calculate total size of jars to download
+		for (int i = 0; i < urlList.length; i++) {
+			urlconnection = urlList[i].openConnection();
+			urlconnection.setDefaultUseCaches(false);
+			if (urlconnection instanceof HttpURLConnection) {
+				((HttpURLConnection) urlconnection).setRequestMethod("HEAD");
+			}
+			
+			fileSizes[i] = urlconnection.getContentLength();
+			
+			long lastModified = urlconnection.getLastModified();
+			String fileName = getFileName(urlList[i]);
+			
+			
+			if (cacheEnabled && lastModified != 0 && 
+					filesLastModified.containsKey(fileName)) {
+				long savedLastModified = ((Long)filesLastModified.get(fileName)).longValue();
+				
+				// if lastModifed time is the same, don't redownload
+				if (savedLastModified == lastModified) {
+					fileSizes[i] = -2; // mark it to not redownload
+				}
+			}
+			
+			if (fileSizes[i] >= 0) {
+				totalSizeDownload += fileSizes[i];
+			}
+			
+			// put key and value in the hashmap
+			filesLastModified.put(fileName, new Long(lastModified));
+			
+			// update progress bar
+			percentage = 5 + (int)(10 * i/(float)urlList.length);
+		}
+	}
+	
 	/**
 	 * Will download the jars from the server using the list of urls
 	 * in urlList, while at the same time updating progress bar
@@ -844,26 +1032,15 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 
 		URLConnection urlconnection;
 		
-		// store file sizes, used for download verification
-		int[] fileSizes = new int[urlList.length];
-		
-		// calculate total size of jars to download
-		for (int i = 0; i < urlList.length; i++) {
-			urlconnection = urlList[i].openConnection();
-			urlconnection.setDefaultUseCaches(false);
-			if (urlconnection instanceof HttpURLConnection) {
-				((HttpURLConnection) urlconnection).setRequestMethod("HEAD");
-			}
-			fileSizes[i] = urlconnection.getContentLength();
-			totalSizeDownload += fileSizes[i];
-		}
-		
-		int initialPercentage = percentage = 10;
+		int initialPercentage = percentage = 15;
 
 		// download each jar
 		byte buffer[] = new byte[65536];
 		for (int i = 0; i < urlList.length; i++) {
 			
+			// skip file if marked as -2 (already downloaded and not changed)
+			if (fileSizes[i] == -2) continue;
+			
 			int unsuccessfulAttempts = 0;
 			int maxUnsuccessfulAttempts = 3;
 			boolean downloadFile = true;
@@ -909,7 +1086,7 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 						// round to two decimal places
 						downloadSpeed = ((int)(downloadSpeed*100))/100f;
 						// set current speed message
-						downloadSpeedMessage = " @ " + downloadSpeed + " KB/sec";
+						downloadSpeedMessage = " - " + downloadSpeed + " KB/sec";
 						// reset downloaded amount
 						downloadedAmount = 0;
 						// reset start time
@@ -1067,6 +1244,10 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 		float increment = (float) 10.0 / urlList.length;
 		// extract all lzma and pack.lzma files
 		for (int i = 0; i < urlList.length; i++) {
+			
+			// if file has not changed, skip it
+			if (fileSizes[i] == -2) continue;
+			
 			percentage = 55 + (int) (increment * (i+1));
 			String filename = getFileName(urlList[i]);
 			
@@ -1102,6 +1283,11 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 	 */
 	protected void extractNatives(String path) throws Exception {
 
+		// if no new native jar was downloaded, no extracting needed
+		if (fileSizes[fileSizes.length-1] == -2) {
+			return;
+		}
+		
 		state = STATE_EXTRACTING_PACKAGES;
 		
 		int initialPercentage = percentage;
@@ -1293,12 +1479,39 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 	 * @param defaultColor Default color to use if no color to load
 	 * @return Color to use
 	 */
-	protected Color getColor(String color, Color defaultColor) {
-		String param_color = getParameter(color);
-		if (param_color != null) {
-			return new Color(Integer.parseInt(param_color, 16));
-		}
-		return defaultColor;
+	protected Color getColor(String param, Color defaultColor) {
+		String color = getParameter(param);
+		
+		if (color == null) return defaultColor;
+		
+		// Check if RGB format
+        if (color.indexOf(",") != -1) {
+            StringTokenizer st = new StringTokenizer(color, ",");
+            
+            // We've got three components for the color
+            try {
+            	return new Color(Integer.parseInt(st.nextToken().trim()), 
+    							 Integer.parseInt(st.nextToken().trim()), 
+    							 Integer.parseInt(st.nextToken().trim()));
+            } catch (Exception e) {
+                // failed to parse
+            	return defaultColor;
+            }
+        }
+        
+        // Check & decode if the color is in hexadecimal color format (i.e. #808000)
+        try {
+        	return Color.decode(color);
+        } catch (NumberFormatException e) {
+        	// ignore exception
+        }
+        
+        // Get the color by name if it exists
+        try {
+        	return (Color)Color.class.getField(color).get(null);
+        } catch (Exception e) {
+        	return defaultColor;
+        }
 	}
 	
 	/**
@@ -1322,7 +1535,7 @@ public class AppletLoader extends Applet implements Runnable, AppletStub {
 	 */
 	protected void fatalErrorOccured(String error, Exception e) {
 		fatalError = true;
-		fatalErrorDescription = "Fatal error occured (" + state + "): " + error;
+		fatalErrorDescription = "This occurred while '" + getDescriptionForState() + "'";
 		System.out.println(fatalErrorDescription);
 		if(e != null) {
 			System.out.println(generateStacktrace(e));
