@@ -37,8 +37,8 @@ package org.lwjgl.util.generator;
  * This class generates the methods in the generated java source files.
  *
  * @author elias_naur <elias_naur@users.sourceforge.net>
- * @version $Revision: 3293 $
- * $Id: JavaMethodsGenerator.java 3293 2010-03-23 12:43:44Z spasi $
+ * @version $Revision: 3355 $
+ * $Id: JavaMethodsGenerator.java 3355 2010-05-27 22:56:29Z spasi $
  */
 
 import com.sun.mirror.apt.*;
@@ -261,9 +261,12 @@ public class JavaMethodsGenerator {
 		// DISABLED: indirect buffer support
 		//printNondirectParameterCopies(writer, method, mode);
 		if (has_result) {
-			if ( method.getAnnotation(GLreturn.class) == null )
-				writer.println("\t\treturn " + Utils.RESULT_VAR_NAME + ";");
-			else
+			if ( method.getAnnotation(GLreturn.class) == null ) {
+				if ( ByteBuffer.class.equals(Utils.getJavaType(result_type)) )
+					writer.println("\t\treturn " + Utils.RESULT_VAR_NAME + ".order(ByteOrder.nativeOrder());"); // safeNewBuffer returns a direct ByteBuffer with BIG_ENDIAN order.
+				else
+					writer.println("\t\treturn " + Utils.RESULT_VAR_NAME + ";");
+			} else
 				Utils.printGLReturnPost(writer, method, method.getAnnotation(GLreturn.class));
 		}
 		writer.println("\t}");
@@ -384,7 +387,10 @@ public class JavaMethodsGenerator {
 				String auto_parameter_name = auto_type_annotation.value();
 				ParameterDeclaration auto_target_param = Utils.findParameter(method, auto_parameter_name);
 				TypeInfo auto_target_type_info = typeinfos_instance.get(auto_target_param);
-				writer.print("(" + auto_parameter_name + ".remaining()");
+				if ( auto_type_annotation.canBeNull() )
+					writer.print("((" + auto_parameter_name + " == null ? 0 : " + auto_parameter_name + ".remaining())");
+				else
+					writer.print("(" + auto_parameter_name + ".remaining()");
 				// Shift the remaining if the target parameter is multityped and there's no AutoType to track type
 				boolean shift_remaining = !hasAnyParameterAutoTypeAnnotation(method, auto_target_param) && Utils.isParameterMultiTyped(auto_target_param);
 				if (shift_remaining) {
@@ -443,8 +449,13 @@ public class JavaMethodsGenerator {
 						writer.print(offset == null ? "0" : offset);
 					} else
 						writer.print("0");
-				} else if ( param.getAnnotation(GLpointer.class) != null ) {
-					writer.print(".getPointer()");
+				} else {
+					GLpointer pointer_annotation = param.getAnnotation(GLpointer.class);
+					if ( pointer_annotation != null ) {
+						if ( pointer_annotation.canBeNull() )
+							writer.print(" == null ? 0 : " + param.getSimpleName());
+						writer.print(".getPointer()");
+					}
 				}
 			}
 		}
@@ -490,7 +501,7 @@ public class JavaMethodsGenerator {
                                         cachedReference != null &&
 					(mode != Mode.BUFFEROBJECT || param.getAnnotation(BufferObject.class) == null) &&
 					param.getAnnotation(Result.class) == null) {
-				writer.print("\t\t" + Utils.CHECKS_CLASS_NAME + ".getReferences(caps).");
+				writer.print("\t\tif ( LWJGLUtil.CHECKS ) " + Utils.CHECKS_CLASS_NAME + ".getReferences(caps).");
                                 if(cachedReference.name().length() > 0) {
                                     writer.print(cachedReference.name());
                                 } else {
