@@ -36,8 +36,8 @@ package org.lwjgl.util.generator;
  * Various utility methods to the generator.
  *
  * @author elias_naur <elias_naur@users.sourceforge.net>
- * @version $Revision: 3299 $
- * $Id: Utils.java 3299 2010-03-31 15:46:16Z spasi $
+ * @version $Revision: 3339 $
+ * $Id: Utils.java 3339 2010-05-07 17:03:36Z spasi $
  */
 
 import java.io.PrintWriter;
@@ -166,7 +166,9 @@ public class Utils {
 		return null;
 	}
 
+	// DISABLED: We always generate indirect methods. (affects OpenAL only at the time of this change)
 	public static boolean isMethodIndirect(boolean generate_error_checks, boolean context_specific, MethodDeclaration method) {
+		/*
 		for (ParameterDeclaration param : method.getParameters()) {
 			if (isAddressableType(param.getType()) || getParameterAutoAnnotation(param) != null ||
 					param.getAnnotation(Constant.class) != null)
@@ -176,6 +178,8 @@ public class Utils {
 			method.getAnnotation(CachedResult.class) != null ||
 			(generate_error_checks && method.getAnnotation(NoErrorCheck.class) == null) ||
 			context_specific;
+		*/
+		return true;
 	}
 
 	public static String getNativeQualifiedName(String qualified_name) {
