@@ -44,8 +44,8 @@ import org.lwjgl.util.generator.*;
  * AL.h version 1.0
  *
  * @author Brian Matzon <brian@matzon.dk>
- * @version $Revision: 3288 $
- * $Id: AL10.java 3288 2010-03-16 17:58:48Z spasi $
+ * @version $Revision: 3380 $
+ * $Id: AL10.java 3380 2010-07-14 13:12:24Z matzon $
  */
 public interface AL10 {
 	/** Bad value */
@@ -250,35 +250,28 @@ public interface AL10 {
 	int AL_FREQUENCY = 0x2001;
 
 	/**
-	 * Sound buffers: frequency, in units of Hertz [Hz].
-	 * This is the number of samples per second. Half of the
-	 * sample frequency marks the maximum significant
-	 * frequency component.
+	 * Sound buffers: The number of bits per sample for the
+	 * data contained in the buffer.
 	 */
 	int AL_BITS = 0x2002;
 
 	/**
-	 * Sound buffers: frequency, in units of Hertz [Hz].
-	 * This is the number of samples per second. Half of the
-	 * sample frequency marks the maximum significant
-	 * frequency component.
+	 * Sound buffers: The number of channels for the data
+	 * contained in the buffer.
 	 */
 	int AL_CHANNELS = 0x2003;
 
 	/**
-	 * Sound buffers: frequency, in units of Hertz [Hz].
-	 * This is the number of samples per second. Half of the
-	 * sample frequency marks the maximum significant
-	 * frequency component.
+	 * Sound buffers: Size in bytes of the buffer data.
 	 */
 	int AL_SIZE = 0x2004;
 
 	/**
-	 * Sound buffers: frequency, in units of Hertz [Hz].
-	 * This is the number of samples per second. Half of the
-	 * sample frequency marks the maximum significant
-	 * frequency component.
+	 * @deprecated This token is a relict of the early OpenAL days and is
+	 * no longer supported. Neither the OpenAL spec nor OpenAL Soft define
+	 * it.
 	 */
+	@Deprecated
 	int AL_DATA = 0x2005;
 
 	/**
